import javax.microedition.lcdui.*;
import java.util.*;

public class Monitor extends Form implements CommandListener  
{ 
    Scheduler scheduler;
    Vector    activeTimers;
    Gauge[]   indicators;
    long      start;
    Timer     timer;

    Monitor(Scheduler scheduler) { 
        super(Locale.MONITOR);
        this.scheduler = scheduler;
        addCommand(Scheduler.QUIT_CMD);
        addCommand(Scheduler.ADD_CMD);
        addCommand(Scheduler.LIST_CMD);
        addCommand(Scheduler.SETTINGS_CMD);
        setCommandListener(this);
        timer = new Timer();
        timer.schedule(new TickTask(), Scheduler.SECOND, Scheduler.SECOND);
        scheduler.display.setCurrent(this);
    }        

    synchronized void tick() { 
        if (indicators != null) { 
            int now = (int)((System.currentTimeMillis() - start)/Scheduler.SECOND);
            for (int i = 0; i < indicators.length; i++) { 
                indicators[i].setValue(now);
            }
        }
    }

    class TickTask extends TimerTask {
        public void run() { 
            tick();
        }
    }

    public void commandAction(Command c, Displayable d)  {
        if (c == Scheduler.QUIT_CMD) { 
            scheduler.quit();
        } else if (c == Scheduler.ADD_CMD) { 
            new TimerList(scheduler, this);
        } else if (c == Scheduler.LIST_CMD) { 
            new ActiveTimerList(scheduler, activeTimers, this);
        } else if (c == Scheduler.SETTINGS_CMD) { 
            new SettingsForm(scheduler, this);
        } else if (c == Scheduler.DISMISS_CMD) {
            scheduler.turnOff();
        }
    }  

    synchronized void refresh() { 
        activeTimers = new Vector();
        Vector timers = scheduler.timers;
        start = System.currentTimeMillis();
        for (int i = 0, n = timers.size(); i < n; i++) { 
            Schedule s = (Schedule)timers.elementAt(i);
            if (s.expiration != 0 && s.expiration > start) { 
                int l = 0, r = activeTimers.size();
                while (l < r) { 
                    int m = (l + r) >> 1;
                    Schedule t = (Schedule)activeTimers.elementAt(m);
                    if (t.expiration < s.expiration) { 
                        l = m+1;
                    } else { 
                        r = m;
                    }
                }
                activeTimers.insertElementAt(s, r);
            }
        }
        deleteAll();
        indicators = new Gauge[activeTimers.size()];
        for (int i = 0; i < indicators.length; i++) { 
            Schedule s = (Schedule)activeTimers.elementAt(i);
            indicators[i] = new Gauge(s.name, false, (int)((s.expiration - start)/1000), 0);
            append(indicators[i]);
        }
    }
}    
